/**
 * Declares the main reasoner API.
 *
 * @author Ivano Bilenchi
 *
 * @file
 */

#ifndef TME_REASONER_H
#define TME_REASONER_H

#include "tme_atomic.h"
#include "tme_iterator.h"
#include "tme_results.h"

TME_BEGIN_DECLS

/// @cond
tme_struct_decl(TmeAxiomProvider);
tme_struct_decl(TmeKB);
tme_struct_decl(TmeReasoner);
tme_struct_decl(TmeSemDesc);
/// @endcond

/**
 * The main reasoner structure, exposing standard and non-standard inference tasks.
 *
 * Non-standard matchmaking services are detailed in "Ruta et al., WIAS 2011":
 * http://sisinflab.poliba.it/publications/2011/RDS11
 *
 * @see http://swot.sisinflab.poliba.it/tinyme
 *
 * @struct TmeReasoner
 */

/// Taxonomy iterator.
TME_ITERATOR_DEF(Taxonomy, TmeAtomicConcept concept);

/// @name Lifecycle

/**
 * Allocates a new reasoner instance.
 *
 * The axiom provider instance is queried in order to populate the internal data structures.
 *
 * @param provider Axiom provider instance.
 * @return Reasoner instance.
 *
 * @public @memberof TmeReasoner
 */
TME_PUBLIC
TmeReasoner* tme_reasoner_alloc(TmeAxiomProvider *provider);

/**
 * Deallocates the reasoner instance.
 *
 * @param reasoner Reasoner instance to deallocate.
 *
 * @public @memberof TmeReasoner
 */
TME_PUBLIC
void tme_reasoner_free(TmeReasoner *reasoner);

/// @name Getters

/**
 * Retrieves the KB instance loaded into the reasoner.
 *
 * @param reasoner Reasoner instance.
 * @return KB loaded into the reasoner.
 *
 * @public @memberof TmeReasoner
 */
TME_PUBLIC
TmeKB* tme_reasoner_get_kb(TmeReasoner *reasoner);

/// @name Ontology reasoning

/**
 * Classifies the ontology loaded into the reasoner.
 *
 * @param reasoner Reasoner instance.
 *
 * @public @memberof TmeReasoner
 */
TME_PUBLIC
void tme_reasoner_classify(TmeReasoner *reasoner);

/**
 * Checks whether the ontology has been classified.
 *
 * @param reasoner Reasoner instance.
 * @return True if the ontology has been classified, false otherwise.
 *
 * @public @memberof TmeReasoner
 */
TME_PUBLIC
bool tme_reasoner_is_classified(TmeReasoner *reasoner);

/**
 * Checks whether the ontology is coherent.
 *
 * @param reasoner Reasoner instance.
 * @return True if the ontology is coherent, false otherwise.
 *
 * @see Moguillansky et al., IBERAMIA 2010
 *      https://cs.uns.edu.ar/~mom/publications/iberamia2010.pdf
 *
 * @public @memberof TmeReasoner
 */
TME_PUBLIC
bool tme_reasoner_is_coherent(TmeReasoner *reasoner);

/// @name Standard reasoning

/**
 * Checks whether the specified individual is consistent with respect to the TBox.
 *
 * @param reasoner Reasoner instance.
 * @param individual Individual whose consistency should be checked.
 * @return True if the individual is consistent, false otherwise.
 *
 * @public @memberof TmeReasoner
 */
TME_PUBLIC
bool tme_reasoner_individual_is_consistent(TmeReasoner *reasoner, TmeAtomicIndividual individual);

/**
 * Determines if a subsumption relation holds between the specified individuals.
 *
 * If the relation holds between two semantic profiles (that is, if one is more general
 * than the other), they are referred to as "subsumer" and "subsumed". The profiles are
 * individuals previously loaded into the reasoner, either because they were present
 * in the root ontology, or because they were loaded using tme_kb_load_individual()
 * or similar functions.
 *
 * If the subsumption relation holds, two conclusions can be drawn:
 *
 * - The subsumer is more general than the subsumed, hence the former (usually a requested
 *   semantic profile) is satisfied by the latter (usually a supplied resource) in every aspect.
 *
 * - The two elements involved in the relation are compatible.
 *
 * If the relation does not hold, tme_reasoner_individual_abduce() may be used
 * to compute what should be hypothesized in the subsumed individual in order to
 * completely satisfy the subsumer individual.
 *
 * @param reasoner Reasoner instance.
 * @param subsumer Subsumer element in the subsumption relation.
 * @param subsumed Subsumed element in the subsumption relation.
 * @return True if the subsumption relation holds, false otherwise.
 *
 * @public @memberof TmeReasoner
 */
TME_PUBLIC
bool tme_reasoner_individual_subsumes(TmeReasoner *reasoner,
                                      TmeAtomicIndividual subsumer, TmeAtomicIndividual subsumed);

/**
 * Checks whether the specified individual is an instance of a certain class.
 *
 * @param reasoner Reasoner instance.
 * @param individual Individual.
 * @param concept Class.
 * @return True if the individual is an instance of the class, false otherwise.
 *
 * @note This is currently incomplete, since the reasoner does not process the ABox.
 *
 * @public @memberof TmeReasoner
 */
TME_PUBLIC
bool tme_reasoner_individual_is_instance(TmeReasoner *reasoner,
                                         TmeAtomicIndividual individual, TmeAtomicConcept concept);

/**
 * Determines if the specified concept is satisfiable with respect to the TBox.
 *
 * @param reasoner Reasoner instance.
 * @param concept Concept whose satisfiability should be verified.
 * @return True if the concept is satisfiable, false otherwise.
 *
 * @public @memberof TmeReasoner
 */
TME_PUBLIC
bool tme_reasoner_concept_is_satisfiable(TmeReasoner *reasoner, TmeAtomicConcept concept);

/**
 * Determines if a subsumption relation holds between the specified concepts.
 *
 * @param reasoner Reasoner instance.
 * @param subsumer Subsumer element in the subsumption relation.
 * @param subsumed Subsumed element in the subsumption relation.
 * @return True if the subsumption relation holds, false otherwise.
 *
 * @see tme_reasoner_individual_subsumes()
 *
 * @public @memberof TmeReasoner
 */
TME_PUBLIC
bool tme_reasoner_concept_subsumes(TmeReasoner *reasoner,
                                   TmeAtomicConcept subsumer, TmeAtomicConcept subsumed);

/**
 * Determines if the specified semantic description is satisfiable with respect to the TBox.
 *
 * @param reasoner Reasoner instance.
 * @param desc Semantic description whose satisfiability should be verified.
 * @return True if the concept is satisfiable, false otherwise.
 *
 * @note The semantic description must be in normal form.
 *
 * @public @memberof TmeReasoner
 */
TME_PUBLIC
bool tme_reasoner_sem_desc_is_satisfiable(TmeReasoner *reasoner, TmeSemDesc const *desc);

/**
 * Determines if a subsumption relation holds between the specified semantic descriptions.
 *
 * @param reasoner Reasoner instance.
 * @param subsumer Subsumer element in the subsumption relation.
 * @param subsumed Subsumed element in the subsumption relation.
 * @return True if the subsumption relation holds, false otherwise.
 *
 * @see tme_reasoner_individual_subsumes()
 * @note Subsumer and subsumed must be in normal form.
 *
 * @public @memberof TmeReasoner
 */
TME_PUBLIC
bool tme_reasoner_sem_desc_subsumes(TmeReasoner *reasoner, TmeSemDesc const *subsumer,
                                    TmeSemDesc const *subsumed);

/**
 * Intersects the semantic descriptions of the specified concepts.
 *
 * @param reasoner Reasoner instance.
 * @param first First concept.
 * @param second Second concept.
 * @return Result of the intersection service.
 *
 * @see tme_reasoner_sem_desc_intersect()
 *
 * @public @memberof TmeReasoner
 */
TME_PUBLIC
TmeSemDesc* tme_reasoner_concept_intersect(TmeReasoner *reasoner, TmeAtomicConcept first,
                                           TmeAtomicConcept second);

/**
 * Intersects the semantic descriptions of the specified individuals.
 *
 * @param reasoner Reasoner instance.
 * @param first First individual.
 * @param second Second individual.
 * @return Result of the intersection service.
 *
 * @see tme_reasoner_sem_desc_intersect()
 *
 * @public @memberof TmeReasoner
 */
TME_PUBLIC
TmeSemDesc* tme_reasoner_individual_intersect(TmeReasoner *reasoner, TmeAtomicIndividual first,
                                              TmeAtomicIndividual second);

/**
 * Intersects the specified semantic descriptions.
 *
 * @param reasoner Reasoner instance.
 * @param first First description.
 * @param second Second description.
 * @return Result of the intersection service.
 *
 * @note The semantic descriptions must be in normal form.
 * @note The returned description is in normal form.
 * @note You are responsible for freeing the intersection result by calling tme_sem_desc_free().
 *
 * @public @memberof TmeReasoner
 */
TME_PUBLIC
TmeSemDesc* tme_reasoner_sem_desc_intersect(TmeReasoner *reasoner, TmeSemDesc const *first,
                                            TmeSemDesc const *second);

/**
 * Iterates over the inferred equivalents of the specified concept.
 *
 * @param reasoner Reasoner instance.
 * @param concept Concept whose equivalents should be iterated.
 * @param iter Taxonomy iterator.
 *
 * @public @memberof TmeReasoner
 */
TME_PUBLIC
void tme_reasoner_iterate_equivalents(TmeReasoner *reasoner, TmeAtomicConcept concept,
                                      TmeTaxonomyIterator *iter);

/**
 * Iterates over the inferred subsumers of the specified concept.
 *
 * @param reasoner Reasoner instance.
 * @param concept Concept whose subsumers should be iterated.
 * @param recursive If true, subsumers are recursively iterated up to the top concept.
 *                  Otherwise, only the direct subsumers are iterated.
 * @param iter Taxonomy iterator.
 *
 * @public @memberof TmeReasoner
 */
TME_PUBLIC
void tme_reasoner_iterate_subsumers(TmeReasoner *reasoner, TmeAtomicConcept concept, bool recursive,
                                    TmeTaxonomyIterator *iter);

/**
 * Iterates over the inferred subsumees of the specified concept.
 *
 * @param reasoner Reasoner instance.
 * @param concept Concept whose subsumees should be iterated.
 * @param recursive If true, subsumees are recursively iterated down to the bottom concept.
 *                  Otherwise, only the direct subsumees are iterated.
 * @param iter Taxonomy iterator.
 *
 * @public @memberof TmeReasoner
 */
TME_PUBLIC
void tme_reasoner_iterate_subsumed(TmeReasoner *reasoner, TmeAtomicConcept concept, bool recursive,
                                   TmeTaxonomyIterator *iter);

/// @name Non-standard reasoning

/**
 * Determines if a supplied resource and a requested semantic profile are compatible.
 *
 * That is, whether the semantic description of the resource does not contain
 * any information in conflict with the request. Resource and request are individuals
 * previously loaded into the reasoner using tme_kb_load_individual() or similar functions.
 *
 * If the compatibility relation between the two individuals holds, then they don't contain
 * any conflicting part. If the relation does not hold, contraction may be used
 * to compute the conflicting part.
 *
 * @param reasoner Reasoner instance.
 * @param resource Resource against which the request is checked for compatibility.
 * @param request Request against which the resource is checked for compatibility.
 * @return True if the compatibility relation holds, false otherwise.
 *
 * @public @memberof TmeReasoner
 */
TME_PUBLIC
bool tme_reasoner_individual_is_compatible(TmeReasoner *reasoner, TmeAtomicIndividual resource,
                                           TmeAtomicIndividual request);

/**
 * Determines if a supplied resource and a requested semantic profile are compatible.
 *
 * @param reasoner Reasoner instance.
 * @param resource Resource against which the request is checked for compatibility.
 * @param request Request against which the resource is checked for compatibility.
 * @return True if the compatibility relation holds, false otherwise.
 *
 * @see tme_reasoner_individual_is_compatible()
 * @note Resource and request must be in normal form.
 *
 * @public @memberof TmeReasoner
 */
TME_PUBLIC
bool tme_reasoner_sem_desc_is_compatible(TmeReasoner *reasoner, TmeSemDesc const *resource,
                                         TmeSemDesc const *request);

/**
 * Performs the Concept Abduction non-standard matchmaking task.
 *
 * Concept Abduction compares a requested semantic profile with a supplied resource.
 * Resource and request are individuals previously loaded into the reasoner
 * using tme_kb_load_individual() or similar functions.
 *
 * @param reasoner Reasoner instance.
 * @param resource Resource.
 * @param request Request.
 * @return Result of the abduction inference service.
 *
 * @note You are responsible for freeing the abduction result by calling tme_abduction_deinit().
 * @see Ruta et al., WIAS 2011 (Section 4, Chapter 2)
 *      (http://sisinflab.poliba.it/publications/2011/RDS11)
 *
 * @public @memberof TmeReasoner
 */
TME_PUBLIC
TmeAbduction tme_reasoner_individual_abduce(TmeReasoner *reasoner, TmeAtomicIndividual resource,
                                            TmeAtomicIndividual request);

/**
 * Performs the Concept Abduction non-standard matchmaking task.
 *
 * @param reasoner Reasoner instance.
 * @param resource Resource.
 * @param request Request.
 * @return Result of the abduction inference service.
 *
 * @note Resource and request must be in normal form.
 * @see tme_reasoner_individual_abduce()
 *
 * @public @memberof TmeReasoner
 */
TME_PUBLIC
TmeAbduction tme_reasoner_sem_desc_abduce(TmeReasoner *reasoner, TmeSemDesc const *resource,
                                          TmeSemDesc const *request);

/**
 * Performs the Concept Bonus non-standard matchmaking task.
 *
 * Concept Bonus compares a requested semantic profile with a supplied resource.
 * Resource and request are individuals previously loaded into the reasoner
 * using tme_kb_load_individual() or similar functions.
 *
 * @param reasoner Reasoner instance.
 * @param resource Resource.
 * @param request Request.
 * @return Result of the bonus inference service.
 *
 * @note You are responsible for freeing the bonus result by calling tme_bonus_deinit().
 * @see Scioscia et al., SRLGIPD18
 *      (http://sisinflab.poliba.it/publications/2018/SRLGIPD18)
 *
 * @public @memberof TmeReasoner
 */
TME_PUBLIC
TmeBonus tme_reasoner_individual_bonus(TmeReasoner *reasoner, TmeAtomicIndividual resource,
                                       TmeAtomicIndividual request);

/**
 * Performs the Concept Bonus non-standard matchmaking task.
 *
 * @param reasoner Reasoner instance.
 * @param resource Resource.
 * @param request Request.
 * @return Result of the bonus inference service.
 *
 * @note Resource and request must be in normal form.
 * @see tme_reasoner_individual_bonus()
 *
 * @public @memberof TmeReasoner
 */
TME_PUBLIC
TmeBonus tme_reasoner_sem_desc_bonus(TmeReasoner *reasoner, TmeSemDesc const *resource,
                                     TmeSemDesc const *request);

/**
 * Performs the Concept Contraction non-standard matchmaking task.
 *
 * Concept Contraction compares a requested semantic profile with a supplied resource.
 * Resource and request are individuals previously loaded into the reasoner
 * using tme_kb_load_individual() or similar functions.
 *
 * @param reasoner Reasoner instance.
 * @param resource Resource.
 * @param request Request.
 * @return Result of the contraction inference service.
 *
 * @note You are responsible for freeing the contraction result by calling tme_contraction_deinit().
 * @see Ruta et al., WIAS 2011 (Section 4, Chapter 2)
 *      (http://sisinflab.poliba.it/publications/2011/RDS11)
 *
 * @public @memberof TmeReasoner
 */
TME_PUBLIC
TmeContraction tme_reasoner_individual_contract(TmeReasoner *reasoner, TmeAtomicIndividual resource,
                                                TmeAtomicIndividual request);

/**
 * Performs the Concept Contraction non-standard matchmaking task.
 *
 * @param reasoner Reasoner instance.
 * @param resource Resource.
 * @param request Request.
 * @return Result of the contraction inference service.
 *
 * @note Resource and request must be in normal form.
 * @see tme_reasoner_individual_contract()
 *
 * @public @memberof TmeReasoner
 */
TME_PUBLIC
TmeContraction tme_reasoner_sem_desc_contract(TmeReasoner *reasoner, TmeSemDesc const *resource,
                                              TmeSemDesc const *request);

/**
 * Performs the Concept Covering non-standard matchmaking task.
 *
 * Concept Covering compares a requested semantic profile with available resources.
 * Request is an OWL individual previously loaded into the reasoner
 * using tme_kb_load_individual() or similar functions.
 *
 * @param reasoner Reasoner instance.
 * @param request Request.
 * @return Result of the covering inference service.
 *
 * @note You are responsible for freeing the covering result by calling tme_composition_deinit().
 * @see Di Noia et al., Concept Covering
 *      (http://sisinflab.poliba.it/publications/2004/DDD04a)
 *
 * @public @memberof TmeReasoner
 */
TME_PUBLIC
TmeComposition tme_reasoner_individual_covering(TmeReasoner *reasoner, TmeAtomicIndividual request);

/**
 * Performs the Concept Covering non-standard matchmaking task.
 *
 * @param reasoner Reasoner instance.
 * @param request Request.
 * @return Result of the covering inference service.
 *
 * @note The request must be in normal form.
 * @see tme_reasoner_individual_covering()
 *
 * @public @memberof TmeReasoner
 */
TME_PUBLIC
TmeComposition tme_reasoner_sem_desc_covering(TmeReasoner *reasoner, TmeSemDesc const *request);

/**
 * Computes the Concept Difference between the specified individuals.
 *
 * @param reasoner Reasoner instance.
 * @param lhs LHS of the difference.
 * @param rhs RHS of the difference.
 * @return Result of the difference service.
 *
 * @note You are responsible for freeing the difference result by calling tme_sem_desc_free().
 *
 * @public @memberof TmeReasoner
 */
TME_PUBLIC
TmeSemDesc* tme_reasoner_individual_difference(TmeReasoner *reasoner, TmeAtomicIndividual lhs,
                                               TmeAtomicIndividual rhs);

/**
 * Computes the Concept Difference between the specified semantic descriptions.
 *
 * @param reasoner Reasoner instance.
 * @param lhs LHS of the difference.
 * @param rhs RHS of the difference.
 * @return Result of the difference service.
 *
 * @note The semantic descriptions must be in normal form.
 * @see tme_reasoner_individual_difference()
 *
 * @public @memberof TmeReasoner
 */
TME_PUBLIC
TmeSemDesc* tme_reasoner_sem_desc_difference(TmeReasoner *reasoner, TmeSemDesc const *lhs,
                                             TmeSemDesc const *rhs);

/**
 * Computes the maximum penalty associated with the specified individual.
 *
 * @param reasoner Reasoner instance.
 * @param individual Individual.
 * @return Maximum penalty.
 *
 * @public @memberof TmeReasoner
 */
TME_PUBLIC
double tme_reasoner_individual_max_penalty(TmeReasoner *reasoner, TmeAtomicIndividual individual);

/**
 * Computes the maximum penalty associated with the specified semantic description.
 *
 * @param reasoner Reasoner instance.
 * @param desc Semantic description.
 * @return Maximum penalty.
 *
 * @public @memberof TmeReasoner
 */
TME_PUBLIC
double tme_reasoner_sem_desc_max_penalty(TmeReasoner *reasoner, TmeSemDesc const *desc);

TME_END_DECLS

#endif // TME_REASONER_H
