/**
 * Declares the available standard reasoning tasks.
 *
 * These functions are generally invoked by the TmeReasoner API,
 * though they are exposed publicly for additional flexibility.
 *
 * @note Unless otherwise stated, semantic descriptions must be unfolded
 *       and normalized prior to calling these functions.
 *
 * @author Ivano Bilenchi
 *
 * @file
 */

#ifndef TME_STANDARD_H
#define TME_STANDARD_H

#include "tme_atomic.h"
#include "tme_cardinality_role.h"
#include "tme_universal_role.h"

TME_BEGIN_DECLS

/// @cond
tme_struct_decl(TmeSemDesc);
/// @endcond

/// @name Normalization

/**
 * Performs the CNF normalization of the specified semantic description.
 *
 * The description is normalized in-place.
 *
 * @param desc Unfolded semantic description to normalize.
 * @param bottom Entity identifier of the bottom concept.
 * @return True if the description is satisfiable, false otherwise.
 *
 * @note The description must have been unfolded beforehand.
 *
 * @public @memberof TmeReasoner
 */
TME_PUBLIC
bool tme_sem_desc_normalize(TmeSemDesc *desc, TmeAtomicConcept bottom);

/// @name Subsumption

/**
 * Determines if a subsumption relation holds between the specified semantic descriptions.
 *
 * @param subsumer Subsumer element in the subsumption relation.
 * @param subsumed Subsumed element in the subsumption relation.
 * @param bottom Entity identifier of the bottom concept.
 * @return True if the subsumption relation holds, false otherwise.
 *
 * @public @memberof TmeReasoner
 */
TME_PUBLIC
bool tme_sem_desc_subsumes(TmeSemDesc const *subsumer,
                           TmeSemDesc const *subsumed, TmeAtomicConcept bottom);

/**
 * Determines if a subsumption relation holds between the specified
 * minimum cardinality restrictions.
 *
 * @param subsumer Subsumer element in the subsumption relation.
 * @param subsumed Subsumed element in the subsumption relation.
 * @return True if the subsumption relation holds, false otherwise.
 *
 * @public @memberof TmeReasoner
 */
TME_PUBLIC
bool tme_gt_role_subsumes(TmeCardinalityRole subsumer, TmeCardinalityRole subsumed);

/**
 * Determines if a subsumption relation holds between the specified
 * minimum cardinality restriction and semantic description.
 *
 * @param subsumer Subsumer element in the subsumption relation.
 * @param subsumed Subsumed element in the subsumption relation.
 * @return True if the subsumption relation holds, false otherwise.
 *
 * @public @memberof TmeReasoner
 */
TME_PUBLIC
bool tme_gt_role_subsumes_sem_desc(TmeCardinalityRole subsumer, TmeSemDesc const *subsumed);

/**
 * Determines if a subsumption relation holds between the specified
 * maximum cardinality restrictions.
 *
 * @param subsumer Subsumer element in the subsumption relation.
 * @param subsumed Subsumed element in the subsumption relation.
 * @return True if the subsumption relation holds, false otherwise.
 *
 * @public @memberof TmeReasoner
 */
TME_PUBLIC
bool tme_lt_role_subsumes(TmeCardinalityRole subsumer, TmeCardinalityRole subsumed);

/**
 * Determines if a subsumption relation holds between the specified
 * maximum cardinality restriction and semantic description.
 *
 * @param subsumer Subsumer element in the subsumption relation.
 * @param subsumed Subsumed element in the subsumption relation.
 * @return True if the subsumption relation holds, false otherwise.
 *
 * @public @memberof TmeReasoner
 */
TME_PUBLIC
bool tme_lt_role_subsumes_sem_desc(TmeCardinalityRole subsumer, TmeSemDesc const *subsumed);

/**
 * Determines if a subsumption relation holds between the specified value restrictions.
 *
 * @param subsumer Subsumer element in the subsumption relation.
 * @param subsumed Subsumed element in the subsumption relation.
 * @param bottom Entity identifier of the bottom concept.
 * @return True if the subsumption relation holds, false otherwise.
 *
 * @public @memberof TmeReasoner
 */
TME_PUBLIC
bool tme_uv_role_subsumes(TmeUniversalRole subsumer, TmeUniversalRole subsumed,
                          TmeAtomicConcept bottom);

/**
 * Determines if a subsumption relation holds between the specified
 * value restriction and semantic description.
 *
 * @param subsumer Subsumer element in the subsumption relation.
 * @param subsumed Subsumed element in the subsumption relation.
 * @param bottom Entity identifier of the bottom concept.
 * @return True if the subsumption relation holds, false otherwise.
 *
 * @public @memberof TmeReasoner
 */
TME_PUBLIC
bool tme_uv_role_subsumes_sem_desc(TmeUniversalRole subsumer, TmeSemDesc const *subsumed,
                                   TmeAtomicConcept bottom);

/**
 * Determines if a subsumption relation holds between the cardinalities of the specified
 * minimum cardinality restrictions.
 *
 * @param subsumer Subsumer element in the subsumption relation.
 * @param subsumed Subsumed element in the subsumption relation.
 * @return True if the subsumption relation holds, false otherwise.
 *
 * @public @related TmeReasoner
 */
#define tme_gt_card_subsumes(subsumer, subsumed) ((subsumed) >= (subsumer))

/**
 * Determines if a subsumption relation holds between the cardinalities of the specified
 * maximum cardinality restrictions.
 *
 * @param subsumer Subsumer element in the subsumption relation.
 * @param subsumed Subsumed element in the subsumption relation.
 * @return True if the subsumption relation holds, false otherwise.
 *
 * @public @related TmeReasoner
 */
#define tme_lt_card_subsumes(subsumer, subsumed) ((subsumed) <= (subsumer))

/// @name Satisfiability

/**
 * Checks the satisfiability of the specified semantic description.
 *
 * @param desc Normalized semantic description.
 * @param bottom Entity identifier of the bottom concept.
 * @return True if the semantic description is satisfiable, false otherwise.
 *
 * @public @memberof TmeReasoner
 */
TME_PUBLIC
bool tme_sem_desc_is_satisfiable(TmeSemDesc const *desc, TmeAtomicConcept bottom);

TME_END_DECLS

#endif // TME_STANDARD_H
