/**
 * Declares the string provider interface.
 *
 * @author Ivano Bilenchi
 *
 * @file
 */

#ifndef TME_STRING_PROVIDER_H
#define TME_STRING_PROVIDER_H

#include "tme_atomic.h"

TME_BEGIN_DECLS

/// @cond
tme_struct_decl(TmeStringProvider);
/// @endcond

/**
 * This structure acts as a string provider for the reasoner.
 *
 * The TmeStringProvider interface must be implemented in order to map entity identifiers
 * to their human-readable representations (usually IRIs).
 *
 * Implementation of this interface must come in the form of function pointers provided to the
 * tme_string_provider_alloc() function. Please refer to its documentation for further info.
 *
 * @see TmeLogger
 *
 * @struct TmeStringProvider
 */

/**
 * Allocates a new string provider.
 *
 * Strings returned by the functions implementing this interface don't need to be dynamically
 * allocated. However, it is important that the should_free out parameter is correctly populated.
 *
 * @param context Context for the subsequent handlers.
 * @param get_string Implements tme_string_provider_get_string().
 * @param get_short_string Implements tme_string_provider_get_short_string().
 * @param context_free Called as part of tme_string_provider_free().
 * @return String provider instance.
 *
 * @public @memberof TmeStringProvider
 */
TME_PUBLIC
TmeStringProvider* tme_string_provider_alloc
    (void *context,
     char const* (*get_string)(void *context, TmeEntityPtr entity_ptr,
                               bool *should_free, size_t *length),
     char const* (*get_short_string)(void *context, TmeEntityPtr entity_ptr,
                                     bool *should_free, size_t *length),
     void (*context_free)(void *context));

/**
 * Deallocates the string provider.
 *
 * @param provider String provider to deallocate.
 *
 * @public @memberof TmeStringProvider
 */
TME_PUBLIC
void tme_string_provider_free(TmeStringProvider *provider);

/**
 * Returns the string associated with the specified entity identifier.
 *
 * @param provider String provider instance.
 * @param entity_ptr Entity identifier.
 * @param [out] should_free True if the string should be freed by the caller, false otherwise.
 * @param [out] length Length of the returned string (excluding the null terminator).
 * @return IRI associated with the entity identifier.
 *
 * @public @memberof TmeStringProvider
 */
TME_PUBLIC
char const* tme_string_provider_get_string(TmeStringProvider const *provider,
                                           TmeEntityPtr entity_ptr, bool *should_free,
                                           size_t *length);

/**
 * Returns the short string associated with the specified entity identifier.
 *
 * The returned string is mainly used for debugging or pretty-printing purposes, and
 * it has no uniqueness requirements (e.g. it can be the fragment of the IRI).
 *
 * @param provider String provider instance.
 * @param entity_ptr Entity identifier.
 * @param [out] should_free True if the string should be freed by the caller, false otherwise.
 * @param [out] length Length of the returned string (excluding the null terminator).
 * @return Short string associated with the entity identifier.
 *
 * @public @memberof TmeStringProvider
 */
TME_PUBLIC
char const* tme_string_provider_get_short_string(TmeStringProvider const *provider,
                                                 TmeEntityPtr entity_ptr, bool *should_free,
                                                 size_t *length);

/**
 * Returns the context passed to the string provider during its allocation.
 *
 * @param provider String provider instance.
 * @return String provider context.
 *
 * @public @memberof TmeStringProvider
 */
TME_PUBLIC
void* tme_string_provider_get_context(TmeStringProvider const *provider);

TME_END_DECLS

#endif // TME_STRING_PROVIDER_H
