/**
 * Declares the subsumers API.
 *
 * @author Ivano Bilenchi
 *
 * @file
 */

#ifndef TME_SUBSUMERS_H
#define TME_SUBSUMERS_H

#include "tme_atomic.h"

TME_BEGIN_DECLS

/// @cond
tme_struct_decl(TmeSubsumers);
/// @endcond

/**
 * Models told subsumption and equivalence relations.
 *
 * A subsumer is said to be "told" when the subsumption relation is explicitly present
 * in the TBox (e.g. if it's not inferred). A "named" or "terminological" equivalence
 * is an equivalence with a named concept (rather than a concept expression).
 *
 * @see TmeAxiomProvider
 *
 * @struct TmeSubsumers
 */

/**
 * Marks the specified concept as a named equivalence.
 *
 * @param subsumers Subsumers instance.
 * @param concept Concept to mark as a named equivalence.
 *
 * @public @memberof TmeSubsumers
 */
TME_PUBLIC
void tme_subsumers_add_named_equivalence(TmeSubsumers *subsumers, TmeAtomicConcept concept);

/**
 * Marks the specified concept as a told subsumer.
 *
 * @param subsumers Subsumers instance.
 * @param concept Concept to mark as a told subsumer.
 *
 * @public @memberof TmeSubsumers
 */
TME_PUBLIC
void tme_subsumers_add_told(TmeSubsumers *subsumers, TmeAtomicConcept concept);

/**
 * Marks the specified concept as a told subsumer, using an associative cache to speed up
 * uniqueness checks.
 *
 * @param subsumers Subsumers instance.
 * @param concept Concept to mark as a told subsumer.
 * @param cache Cache for uniqueness checks.
 *
 * @note If cache is NULL, it falls back to regular tme_subsumers_add_told.
 *
 * @public @memberof TmeSubsumers
 */
TME_PUBLIC
void tme_subsumers_add_told_aided(TmeSubsumers *subsumers, TmeAtomicConcept concept,
                                  UHash(TmeEntityTable) *cache);

/**
 * Marks the presence of an equivalence relation with a concept expression.
 *
 * @param subsumers Subsumers instance.
 *
 * @public @memberof TmeSubsumers
 */
TME_PUBLIC
void tme_subsumers_mark_equivalence_with_expression(TmeSubsumers *subsumers);

/**
 * Allows populating the subsumers data structure in a single function call.
 *
 * @param subsumers Subsumers instance.
 * @param told Told subsumers.
 * @param equivalents Named equivalents.
 * @param equivalence_with_expression Marks the presence of an equivalence relation
 *                                    with a concept expression.
 *
 * @note Vectors passed to this function are directly assigned to the internal data structures,
 *       so you must not deallocate them.
 *
 * @public @memberof TmeSubsumers
 */
TME_PUBLIC
void tme_subsumers_populate(TmeSubsumers *subsumers, Vector(TmeAtomicConcept) *told,
                            Vector(TmeAtomicConcept) *equivalents,
                            bool equivalence_with_expression);

TME_END_DECLS

#endif // TME_SUBSUMERS_H
