/**
 * Declares the value restriction data structure and related API.
 *
 * @author Ivano Bilenchi
 *
 * @file
 */

#ifndef TME_UNIVERSAL_ROLE_H
#define TME_UNIVERSAL_ROLE_H

#include "tme_atomic.h"

TME_BEGIN_DECLS

/// @cond
tme_struct_decl(TmeSemDesc);
/// @endcond

/**
 * Models value restrictions.
 *
 * @see TmeSemDesc
 */
typedef tme_struct(TmeUniversalRole) {

    /// Role this restriction refers to.
    TmeAtomicRole atomic_role;

    /// Filler of this restriction.
    TmeSemDesc *filler;

} TmeUniversalRole;

/// Vector of value restrictions.
VECTOR_DECL_EQUATABLE_SPEC(TmeUniversalRole, TME_PUBLIC)

/**
 * Copies the specified value restriction.
 *
 * @param role The restriction to copy.
 * @return Copied restriction.
 *
 * @note The copy is deep (e.g. fillers are recursively copied).
 *
 * @public @memberof TmeUniversalRole
 */
TME_PUBLIC
TmeUniversalRole tme_uv_role_copy(TmeUniversalRole role);

/**
 * Equality function for value restrictions.
 *
 * @param role LHS of the equality relation.
 * @param other RHS of the equality relation.
 * @return True if the equality relation holds, false otherwise.
 *
 * @note This function has no semantic meaning, as it also accounts for the order in which
 *       concepts and roles appear in the filler. It's mainly meant to be used in data structures
 *       that require their elements to be equatable.
 *
 * @public @memberof TmeUniversalRole
 */
TME_PUBLIC
bool tme_uv_role_equals(TmeUniversalRole role, TmeUniversalRole other);

/**
 * De-initializes the specified value restriction, releasing memory held by its filler.
 *
 * @param role The restriction to de-initialize.
 *
 * @public @related TmeUniversalRole
 */
#define tme_uv_role_deinit(role) (tme_sem_desc_free((role).filler))

TME_END_DECLS

#endif // TME_UNIVERSAL_ROLE_H
