/**
 * Defines the numeric entity identifiers used throughout the API.
 *
 * @author Ivano Bilenchi
 *
 * @file
 */

#ifndef TME_ATOMIC_H
#define TME_ATOMIC_H

#include "tme_std.h"
#include "tme_base_types.h"

TME_BEGIN_DECLS

/// @cond
tme_struct_decl(TmeSemDesc);
/// @endcond

/// @name Primitive types

/**
 * Type of a TmeEntityPtr variable.
 *
 * @def TME_ENTITY_PTR_TYPE
 */

/**
 * Maximum value of a TmeEntityPtr variable.
 *
 * @def TME_ENTITY_PTR_MAX
 */

/**
 * Format string for TmeEntityPtr variables.
 *
 * @def TME_ENTITY_PTR_FMT
 */

#ifndef TME_ENTITY_PTR_TYPE
    #define TME_ENTITY_PTR_TYPE uintptr_t
    #define TME_ENTITY_PTR_MAX UINTPTR_MAX
    #define TME_ENTITY_PTR_FMT PRIuPTR
#endif

/**
 * Alignment of a TmeEntityPtr variable.
 *
 * If you're using memory addresses as entity pointers, set this value to their alignment
 * for slightly improved performance.
 */
#ifndef TME_ENTITY_PTR_ALIGNMENT
    #define TME_ENTITY_PTR_ALIGNMENT 1U
#endif

/**
 * Unique identifier for entities.
 *
 * The reasoner does not need to know the human-readable representation of entities
 * (such as IRIs). This is exploited by explicitly dropping any reference to such representations
 * and exclusively relying on numeric identifiers. Of course, some form of mapping between
 * identifiers and textual representations should be kept elsewhere and used to implement
 * the TmeStringProvider interface.
 *
 * @note Identifiers must be unique across all entities (concepts, roles and individuals).
 */
typedef TME_ENTITY_PTR_TYPE TmeEntityPtr;

/// Unique identifier for concepts.
#define TmeAtomicConcept TmeEntityPtr

/// Unique identifier for individuals.
#define TmeAtomicIndividual TmeEntityPtr

/// Unique identifier for roles.
#define TmeAtomicRole TmeEntityPtr

/// Vector of entity identifiers.
VECTOR_DECL_COMPARABLE_SPEC(TmeEntityPtr, TME_PUBLIC)

/// Hash table of (entity identifier, semantic description) pairs.
UHASH_DECL_SPEC(TmeEntityTable, TmeEntityPtr, TmeSemDesc*, TME_PUBLIC)

/// @name Equality and hashing

/**
 * Equality function for entity pointers.
 *
 * @param a LHS of the equality.
 * @param b RHS of the equality.
 * @return True if the equality relation holds, false otherwise.
 */
#define tme_entity_ptr_equals(a, b) uhash_identical(a, b)

/**
 * Hash function for entity pointers.
 *
 * @param e Entity pointer whose hash should be computed.
 * @return Hash value for the entity pointer.
 */
#if TME_ENTITY_PTR_MAX <= 0xff
    #define tme_entity_ptr_hash(e) uhash_int8_hash((uint8_t)((e) / TME_ENTITY_PTR_ALIGNMENT))
#elif TME_ENTITY_PTR_MAX <= 0xffff
    #define tme_entity_ptr_hash(e) uhash_int16_hash((uint16_t)((e) / TME_ENTITY_PTR_ALIGNMENT))
#elif TME_ENTITY_PTR_MAX <= 0xffffffff
    #define tme_entity_ptr_hash(e) uhash_int32_hash((uint32_t)((e) / TME_ENTITY_PTR_ALIGNMENT))
#else
    #define tme_entity_ptr_hash(e) uhash_int64_hash((uint64_t)((e) / TME_ENTITY_PTR_ALIGNMENT))
#endif

TME_END_DECLS

#endif // TME_ATOMIC_H
